unit draw;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, engine;

type
  TdrawForm = class(TForm)
    zoomBar: TScrollBar;
    velocityLabel: TLabel;
    compButton: TButton;
    StopButton: TButton;
    procedure FormPaint(Sender: TObject);
    procedure zoomBarChange(Sender: TObject);
    procedure FormActivate(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure compButtonClick(Sender: TObject);
    procedure StopButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  drawForm: TdrawForm;

  procedure traceElectron;
  
implementation


{$R *.DFM}

procedure traceElectron;
var
  mx,my,x,y,i: integer;
  Ex :TComplexN;
  posAnl,k:extended;
begin
  mx:= drawForm.width div 2;
  my:= drawForm.height div 2;
  stop:= false;
  k:=1;
  if emitterDia>acclrDia then k:=-1;
  
  for i:=1 to emittedEl do begin
   with electron do begin
    posAnl:=(random(1000)/1000)*2*pi;
    expToAdd(emitterDia,posAnl,pos);
    expToAdd(emissionVelocity*k,posAnl+((random(500)-250)/500)*(pi/2),v);
   end;
   repeat
    getE(Ex,electron);
    intStep(electron,Ex);
    x:= trunc(electron.pos.r*zoom);
    y:= trunc(electron.pos.i*zoom);
    drawForm.canvas.pixels[mx+x,my-y]:= clBlack;

    application.ProcessMessages;
   until collision(anl(electron.pos)) or Stop;
   if stop then break;
  end;
  drawForm.velocityLabel.caption:= 'Velocity of e-: '+floatToStr(absc(electron.v));
end;

procedure TdrawForm.FormPaint(Sender: TObject);
var
  mx,my,x1,y1,x2,y2,i: integer;

begin
  { draw HEC }
  drawForm.canvas.brush.style:=bsClear;
  mx:= drawForm.width div 2;
  my:= drawForm.height div 2;
  { draw emitter }
  drawForm.canvas.pen.color:= clRed;
  x1:= mx - round((emitterDia*zoom));
  y1:= my - round((emitterDia*zoom));
  x2:= mx + round((emitterDia*zoom));
  y2:= my + round((emitterDia*zoom));
  if (x1+x2+y1+y2)<32768 then
    drawForm.canvas.ellipse(x1,y1,x2,y2);
  { draw acceletartor }
  drawForm.canvas.pen.color:= clBlack;
  x1:= mx - round((acclrDia*zoom));
  y1:= my - round((acclrDia*zoom));
  x2:= mx + round((acclrDia*zoom));
  y2:= my + round((acclrDia*zoom));
  if (x1+x2+y1+y2)<32768 then
    drawForm.canvas.ellipse(x1,y1,x2,y2);
  { draw collectors }
  drawForm.canvas.pen.color:= clBlack;
  for i:=0 to coldefCount-1 do begin
    x1:= mx + round(cos(getAcclrAngle(i))*startCDia*zoom);
    y1:= my - round(sin(getAcclrAngle(i))*startCDia*zoom);
    x2:= mx + round(cos(getAcclrAngle(i))*endCDia*zoom);
    y2:= my - round(sin(getAcclrAngle(i))*endCDia*zoom);
    drawForm.canvas.moveTo(x1,y1);
    drawForm.canvas.lineTo(x2,y2);
  end;
  { draw deflectors }
  drawForm.canvas.pen.color:= clBlue;
  for i:=0 to coldefCount-1 do begin
    x1:= mx + round(cos(getDeflectAngle(i))*startCDia*zoom);
    y1:= my - round(sin(getDeflectAngle(i))*startCDia*zoom);
    x2:= mx + round(cos(getDeflectAngle(i))*endCDia*zoom);
    y2:= my - round(sin(getDeflectAngle(i))*endCDia*zoom);
    drawForm.canvas.moveTo(x1,y1);
    drawForm.canvas.lineTo(x2,y2);
  end;
  velocityLabel.caption:= 'Velocity of e-: '+floatToStr(emissionVelocity);
end;

procedure TdrawForm.zoomBarChange(Sender: TObject);
begin
  drawForm.refresh;
  FormPaint(self);
  zoom:= zoomBar.position+10;
  zoom:= zoom*100;
end;

procedure TdrawForm.FormActivate(Sender: TObject);
begin
  drawForm.refresh;
  FormPaint(self);
end;

procedure TdrawForm.FormResize(Sender: TObject);
begin
  drawForm.refresh;
  FormPaint(self);
end;

procedure TdrawForm.compButtonClick(Sender: TObject);
begin
  compButton.enabled:=false;
  traceElectron;
  compButton.enabled:=true;
end;

procedure TdrawForm.StopButtonClick(Sender: TObject);
begin
  stop:= true;
end;

procedure TdrawForm.FormCreate(Sender: TObject);
begin
  zoomBarChange(self);
end;

end.
