unit engine;

interface

const qeme = 1.8E+11;

type
   TComplexN = record
                r,i:real;
               end;
   TParticle = record
                 pos:TComplexN;
                 v:TComplexN;
               end;
var
  emitterDia, acclrDia, startCDia, endCDia,
  acclrV, deflectV,ts,emissionVelocity,emissionEnergy: extended;
  emittedEl:integer;
  zoom: extended;
  coldefCount: integer;
  electron:TParticle;
  stop: boolean;
  sens: extended;

function getDeflectAngle(n:integer):real;
function getAcclrAngle(n:integer):real;
function anl(N:TComplexN):real;
function absc(N:TComplexN):real;
procedure getE(var Ex:TComplexN;ePos:TParticle);
procedure intStep(var el:TParticle;Ex:TComplexN);
procedure expToAdd(Nabs,angle:extended;var N:TComplexN);
procedure setEmissionVelo;
function collision(anl:extended):boolean;



implementation


procedure setEmissionVelo;
begin
  emissionVelocity:= sqrt(2*emissionEnergy*qeme);
end;

function getDeflectAngle(n:integer):real;
begin
  result:= ((2*pi*n)/coldefCount) + (pi/colDefCount);
end;

function getAcclrAngle(n:integer):real;
begin
  result:= ((pi/colDefCount)*n)*2;
end;

function collision(anl:extended):boolean;
var
  i: integer;
begin
  result:= false;
  for i:=0 to (2*coldefCount)-1 do begin
    if abs(anl-((pi/coldefCount)*i))<sens then begin
      result:= true;
      break;
    end;
  end;
end;

function getAcclrE(r:extended):extended;
begin
  if emitterDia<acclrDia then
    result:= acclrV/(ln(acclrDia/emitterDia)*r)
  else
    result:= (-1)*acclrV/(ln(emitterDia/acclrDia)*r);
end;

function getDeflectE(r:extended):extended;
begin
  if (startCDia<endCDia) and (r>startCDia) and (r<endCDia) then
    result:= deflectV/((pi*r)/colDefCount)
  else
    result:=0;
end;

procedure getE(var Ex:TComplexN;ePos:TParticle);
var
  posAngle,posDia,dirAnl:extended;
  Eacclr,Edeflect:TComplexN;
  x: integer;
begin
  { get position of electron in exp form }
  posAngle:= anl(ePos.pos);
  posDia:= absc(ePos.pos);
  { locate electron vs. position of deflector }
  x:= trunc(posAngle/(pi/coldefCount));
  if (x mod 2)=0 then
    dirAnl:= (-1)*pi/2
  else
    dirAnl:= pi/2;

  { make deflection E in add form }
  expToAdd(getDeflectE(posDia),posAngle+dirAnl,Edeflect);
  expToAdd(getAcclrE(posDia),posAngle,Eacclr);
  Ex.i:= Edeflect.i+Eacclr.i;
  Ex.r:= Edeflect.r+Eacclr.r;
end;

procedure intStep(var el:TParticle;Ex:TComplexN);
var
  dv:TComplexN;
begin
  { }
  dv.i:= qeme*ex.i*ts;
  dv.r:= qeme*ex.r*ts;
  { acceleration+deflection of electron }
  el.v.i:= el.v.i + dv.i;
  el.v.r:= el.v.r + dv.r;
  { movement }
  el.pos.i:= el.pos.i + el.v.i*ts;
  el.pos.r:= el.pos.r + el.v.r*ts;
end;

{ MATH }

{ angle of |X| }
function anl(N:TComplexN):real;
begin
  if N.i>0 then begin
    if N.r=0 then
      result:= pi/2
    else
      if N.r>0 then
        result:= arctan(N.i/N.r)
      else
        result:= pi+arctan(N.i/N.r);
  end
  else begin
    if N.r=0 then
      result:= (3/2)*pi
    else
      if N.r>0 then
        result:= 2*pi+arctan(N.i/N.r)
      else
        result:= pi+arctan(N.i/N.r)
  end;
end;

function absc(N:TComplexN):real;
begin
  result:= sqrt(sqr(N.i)+sqr(N.r));
end;

procedure expToAdd(Nabs,angle:extended;var N:TComplexN);
begin
  N.r:= Nabs*cos(angle);
  N.i:= Nabs*sin(angle);
end;

end.
